export class DomHelper {
    static *getAllChildNodes(element, includeShadowDom = false, ignore) {
        if (!ignore || ignore.indexOf(element) < 0) {
            if (element.children) {
                for (const node of element.children) {
                    yield node;
                    const childs = DomHelper.getAllChildNodes(node, includeShadowDom, ignore);
                    for (const cnode of childs) {
                        yield cnode;
                    }
                }
            }
            if (includeShadowDom && element.shadowRoot != null) {
                yield element.shadowRoot;
                const childs = DomHelper.getAllChildNodes(element.shadowRoot, includeShadowDom, ignore);
                for (const cnode of childs) {
                    yield cnode;
                }
            }
        }
        return null;
    }
    static removeAllChildnodes(node, selector) {
        if (!selector) {
            for (let c = node.firstChild; c !== null; c = node.firstChild) {
                node.removeChild(c);
            }
        }
        else {
            const elements = node.querySelectorAll(selector);
            for (const e of elements) {
                if (e.parentNode == node)
                    node.removeChild(e);
            }
        }
    }
    static nodeIndex(node) {
        let i = 0;
        while ((node = node.previousSibling) != null)
            i++;
        return i;
    }
    static childIndex(node) {
        for (let i = 0; i < node.parentElement.children.length; i++)
            if (node.parentElement.children[i] == node)
                return i;
        return -1;
    }
    static getHost(node) {
        return node.getRootNode()?.host;
    }
    static nodeIsChildOf(node, parentNode) {
        while (node.parentElement) {
            if (node.parentElement == parentNode)
                return true;
            node = node.parentElement;
        }
        return false;
    }
    static findParentNodeOfWithLocalName(element, localName) {
        let el = element;
        while (true) {
            if (el.parentElement != null) {
                el = el.parentElement;
            }
            else if (el.parentNode != null && el.parentNode.host != null) {
                el = el.parentNode.host;
            }
            else {
                break;
            }
            if (el.localName == localName) {
                return el;
            }
        }
        return null;
    }
    static findParentNodeOfType(element, type) {
        let el = element;
        while (true) {
            if (el.parentElement != null) {
                el = el.parentElement;
            }
            else if (el.parentNode != null && el.parentNode.host != null) {
                el = el.parentNode.host;
            }
            else {
                break;
            }
            if (el instanceof type) {
                return el;
            }
        }
        return null;
    }
}
